%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Preliminary program: This program should be run first.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Setting up the simulation.');

%% Set parameters for simulation
alpha = 0.1; % significance level
display_info = 1; % verbose on/off

%% Detect Simulation Type
directory_name = pwd; % record directory name

% directory should be named SOMENAME_x, where x = denotes coordinate of interest for inference (1 or 2).
coordinate = str2double(directory_name(end)); % Grab x, last character from directory name.
assert(coordinate==1 || coordinate==2);

%% Primitives of DGP
theta0 = [0.3,0.5,0,0,0.05]; % true parameter of interest. In the paper is theta0 = (theta1,theta2,beta2,beta3,beta1).
dimSX = size(theta0,2)-1; % dimension of X.
PX = ones(dimSX,1)/dimSX; % P(X=x), X is discrete uniform.
prob10 = 0.6; % prob of P(A_1=1,A_2=0) when there is multiplicity.

%% Grid of values of theta according to H0;
theta_delta = 0.005;
if coordinate == 1
    theta_H0_list = (0.10:theta_delta:0.50)';
else % coordinate = 2.
    theta_H0_list = (0.35:theta_delta:0.75)';
end

%% Define boundaries of the Id Set. These numbers were numerically calculated by Ivan
if coordinate == 1
    trueH0  = (theta_H0_list>=0.23).*(theta_H0_list<=0.36);
else % coordinate = 2.
    trueH0  = (theta_H0_list>=0.455).*(theta_H0_list<=0.555);
end

% sample and bootstrap sample sizes;
n = 1000; % sample size.
R = 300; % bootstrap sample size, i.e., the simulated quantiles are the result of R simulated values.
MC_size = 2000; % MC size, i.e., empirical coverage is the result of MC_size tests.

%% Tuning parameters for inference;
bn_list =  round([n/4;n^(2/3)]); % subsampling tuning parameter.
C_list = [0.8;1]; % constant in the paper;
kappa_list = C_list*sqrt(log(n));  % GMS Thresholding parameter, preferred choice Eq. 4.4 in Andrews and Soares (2010).

%% Parameters for optimization;
tolerance = 0.0001; % define what we mean by "close enough".

% options for minimization.
options = optimset('Display','off','Algorithm','interior-point'); % (this is slower).
% options = optimset('Display','off','Algorithm','active-set'); % (this is faster).

random_seed_start = 1; % Arbitrary start for the seed;
random_seed_list  = random_seed_start+(1:1:MC_size); % list or random seeds.

disp('Save all information in the BASE directory');
% Create directory to save this information;
mkdir results;
cd results
matrix_reject_test_DR = NaN(MC_size,size(theta_H0_list,1),size(kappa_list,1));
matrix_reject_test_PR = NaN(MC_size,size(theta_H0_list,1),size(kappa_list,1));
matrix_reject_test_MR = NaN(MC_size,size(theta_H0_list,1),size(kappa_list,1));
matrix_reject_test_SS = NaN(MC_size,size(theta_H0_list,1),size(bn_list,1));
save results matrix_reject_test_*;
cd ..

mkdir BASE
cd BASE
save Data.mat alpha display_info coordinate theta0 dimSX PX prob10 theta_H0_list trueH0 n R bn_list C_list kappa_list tolerance options MC_size random_seed_list;
cd ..