%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Read Results programs. Once the MC_size simulations are completed, it
%   reads results.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
clc

% Load basic information;
cd BASE
load Data.mat MC_size bn_list kappa_list theta_H0_list trueH0 alpha
cd ..

% Load completed MC results;
cd results
load results
cd ..

% Check if all simulations were completed;
if sum(sum(isnan(matrix_reject_test_SS)))>0
    disp('Simulations are incomplete. Here is a count of incomplete simulations for each coordinate of theta.');
    disp([theta_H0_list,(squeeze(sum(isnan(matrix_reject_test_SS(:,:,1)))))']);
end

% Compute average rejection rate for each inference method;
results_MR = squeeze(nanmean(matrix_reject_test_MR));
results_SS = squeeze(nanmean(matrix_reject_test_SS));
results_PR = squeeze(nanmean(matrix_reject_test_PR));
results_DR = squeeze(nanmean(matrix_reject_test_DR));

% Close all graphs;
close

% Create figure 1;
plot(theta_H0_list,results_MR(:,1),'-r')
hold on
plot(theta_H0_list,results_MR(:,2),'--m')
plot(theta_H0_list,results_SS(:,1),'--b')
plot(theta_H0_list,results_SS(:,2),'-.g')
plot(theta_H0_list(trueH0==1),ones(size(theta_H0_list(trueH0==1)))*alpha,'*k')
grid on
legend('Test MR1','Test MR2','Test SS1','Test SS2')

% Save figure 1
cd results
saveas(gcf,'figure1.png')
cd ..

% Create figure 2
axis([0.15 0.26 0 1])

% Save figure 2
cd results
saveas(gcf,'figure2.png')
cd ..

cd ..